@@ -72,6 +72,8 @@ def get_styles( o, restyle ):
 
     return styles
 
+def ci_dict_get( d, k):
+    pass
 
 def gpml2svg(
     gpml, 
@@ -80,6 +82,10 @@ def gpml2svg(
         'HMDB': 'http://www.hmdb.ca/metabolites/%s',
         'WikiPathways': 'http://wikipathways.org/index.php/Pathway:%s',
         'Entrez Gene': 'http://www.ncbi.nlm.nih.gov/gene/%s',
+        'PubChem-compound': 'http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=%s',
+        'Uniprot/TrEMBL': 'http://www.uniprot.org/uniprot/%s',
+        'KEGG Orthology': 'http://www.genome.jp/dbget-bin/www_bget?ko+%s',
+        'Kegg Compound': 'http://www.kegg.jp/entry/%s',
     },
     xrefs_synonyms = {}, # A conversion dictionary to generate additional XRefs
     xrefs_classes = ['HMDB','Entrez Gene'], # A list of XRefs to use to generate data node classes (post-coloring/transformations)
@@ -100,9 +106,11 @@ def gpml2svg(
 
     # Get root page canvas size, pathway name and copyright data from the header
     text = {}
+    metadata = {}
     y = 0
     for k in ['Name', 'Last-Modified', 'Organism', 'License']:
         if k in tree.attrib:
+            metadata[k] = tree.attrib[k]
             y += 15
             text[k] = (tree.attrib[k], y) 
     
@@ -362,10 +370,14 @@ def gpml2svg(
         path = []
         xp, yp = False, False
         for p in ps:
-                
-            x = float(p.attrib['X'])
-            y = float(p.attrib['Y'])
 
+            try:                
+                x = float(p.attrib['X'])
+                y = float(p.attrib['Y'])
+            except: # wtf?
+                x = float(p.attrib['x'])
+                y = float(p.attrib['y'])
+                            
             if 'GraphRef' in p.attrib:
                 graphref = p.attrib['GraphRef']
                 d = get_direction(xp, yp, x, y)
@@ -374,7 +386,7 @@ def gpml2svg(
             else:
                 graphref = False
 
-            path.append( (x, y, graphref) )
+            path.append( (x, y, graphref, '-') )
             xp, yp = x, y
             
         
@@ -406,7 +418,9 @@ def gpml2svg(
 
         anchor = []
         for a in anchors:
-            anchor.append( a.attrib['GraphId'] )
+            if 'GraphId' in a.attrib:
+                anchor.append( a.attrib['GraphId'] )
+        
 
         idata = {
                 'path': path,
@@ -445,7 +459,7 @@ def gpml2svg(
         gdir = dirs[ get_direction( path[0][0], path[0][1], path[-1][0], path[-1][1]) ]
         
         # START: Draw out from the edge of origin box
-        [xo, yo, graphref] = path[0]
+        [xo, yo, graphref, d] = path[0]
         inpath = [ (xo, yo) ]
         if graphref in graphids:
             data = graphids[ graphref ]         
@@ -457,7 +471,7 @@ def gpml2svg(
                 path[0] = (xo, yo, graphref, d) # Amend the path, so we can just iterate through the middle and look left-right for all points
 
         # END: Draw out from edge of destination box
-        [xd, yd, graphref] = path[-1]
+        [xd, yd, graphref, d] = path[-1]
         outpath = [ (xd, yd) ]
         if graphref in graphids:                
             # END: Draw out from the edge of destination box
@@ -498,12 +512,14 @@ def gpml2svg(
                     inpath.append( 
                         {
                             'H' : (path[n][0], path[n-1][1]),
+                            '-' : (path[n][0], path[n][1]), # May be angled
                             'V' : (path[n-1][0], path[n][1])
                         }[ path[n-1][3] ] )
 
                     outpath.append( 
                         {
                             'H' : (path[n][0], path[n+1][1]),
+                            '-' : (path[n][0], path[n][1]), # May be angled
                             'V' : (path[n+1][0], path[n][1])
                         }[ path[n+1][3] ] )
                     pass
@@ -512,7 +528,8 @@ def gpml2svg(
             # Draw straight hook
             options = {
                 'H' : (path[-1][0], path[0][1]),
-                'V' : (path[0][0], path[-1][1])
+                'V' : (path[0][0], path[-1][1]),
+                '-' : (path[0][0], path[1][1]) # May be angled
                 }
             di = path[0][3] if path[0][3] in options else gdir #H/V
             inpath.append( options[di] )
@@ -531,7 +548,7 @@ def gpml2svg(
         # if matching, draw through that point to the end of current
 
 
-    metadata = {
+    render_data = {
         'page': page,
         'text': text,
         'data_nodes': data_nodes_svg,
@@ -545,7 +562,7 @@ def gpml2svg(
     }
 
     template = templateEngine.get_template( os.path.join( scriptdir,'gpml2svg.svg') )
-    return template.render( metadata )
+    return template.render( render_data ), metadata # return additional metadata
 
 def main():
     # Run from command line
@@ -567,11 +584,13 @@ def main():
     gpml = f.read().decode('utf8')
     f.close()
 
-    svg = gpml2svg( gpml )
+    svg, metadata = gpml2svg( gpml )
 
     if options.outfile is None:
         options.outfile = options.file.replace('gpml','svg')
         
+    print "Rendered pathway '%s' to SVG, saving as %s" % (metadata['Name'], options.outfile)
+
     o = codecs.open(options.outfile,'w','utf-8')
     o.write( svg )
     o.close()
